;//******************************************************************************
;//* PROJECT:		Vertical Blank Interrupt Example
;//* AUTHOR:		Adrian Brown
;//* DATE:			21st May 2000
;//******************************************************************************
;//* HISTORY:
;//* 21/05/00		[APB]		Created initial example
;//******************************************************************************

INCLUDE			"Hardware.inc"

;// Wait for VRAM Macros
WaitForVRAM:	MACRO
				ld		a, [rSTAT] 
				and		a, STATF_OAM
				jr		nz, @-4
				ENDM

WaitForVRAMStore: MACRO
				push	af
				ld		a, [rSTAT] 
				and		a, STATF_OAM
				jr		nz, @-4
				pop		af
				ENDM

;// 
;// RGB Macro, allows using 0-255 range
;//

RGB:			MACRO
				DW		((\3 >> 3) << 10) + ((\2 >> 3) << 5) + (\1 >> 3)
				ENDM
				
;//******************************************************************************
;//*						INTERRUPTS HANDLERS									*
;//******************************************************************************

				SECTION "Interrupts", HOME[$0040]

				;// Since the space for the handlers is only 8 bytes - best jump to another location
VBInt:			jp		VBHandler

;//******************************************************************************
;//*						CARTRIDGE HEADER									*
;//******************************************************************************

				SECTION "Main", HOME[$0100]

				;//* [$0100] Program Start 0 Jump to main entry point
				nop
				jp		Initialise

				;//* [$0104] Cartridge Header - Nintendo logo etc
				NINTENDO_LOGO

				;// Cartridge name
				DB		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

				;// License code
				DB		0, 0

				;// Game type (0 = gameboy, 3 = super gameboy)
				DB		0

				;// Cartridge type
				DB		CART_ROM

				;// Rom Size
				DB		CART_ROM_256K

				;// Ram Size
				DB		CART_RAM_NONE

				;// Manufacturer code
				DW		0

				;// Version Number
				DB		0

				;// Complement check
				DB		$0a

				;// Checksum
				DW		0

;//******************************************************************************
;//*								MAIN ENTRY POINT							*
;//******************************************************************************

Initialise:		;// Set the stack pointer
				ld		sp, Stack

				;// Now disable interrupts and screen
				di

				;// Remember to wait for the VBlank
.WaitVB:		ld		a, [rLY]
				cp		a, 144
				jr		nz, .WaitVB

				;// Now disable everything
				ld		a, LCDCF_OFF
				ld		[rLCDC], a

				;// Clear the variables
				xor		a							
				ld		hl, MemoryStart				
				ld		bc, MemoryEnd - MemoryStart	
				call	Memory_Clear

				;// Clear video ram - we really should use a routine which checks for the video ram to be accessable
				;// but as the screen is off it will be - first clear the 1st bank

				ld		a, 1
				ld		[rVBK], a

				ld		hl, $8000
				ld		bc, $2000
				xor		a, a
				call	Memory_Clear

				;// Now clear bank 0
				xor		a, a
				ld		[rVBK], a

				ld		hl, $8000
				ld		bc, $2000
				call	Memory_Clear

				;// Upload the tile data
				ld		hl, TileData
				ld		de, $9000
				ld		c, 12
				call	MoveToVideoRam

				;// Set the background palette
				ld		hl, Palette
				xor		a, a
				call	SetPalette

				;// Enable the vertical blank interrupt
				ld		a, IEF_VBLANK
				ld		[rIE], a

				;// Enable the screen display
				ld		a, LCDCF_ON | LCDCF_WINOFF | LCDCF_BG8800 | LCDCF_BGON | LCDCF_BG9800 | LCDCF_OBJ8 | LCDCF_OBJOFF
				ld		[rLCDC], a

				;// Now call the entry point of the game after enabling the interrupts
				ei

				;//
				;// Main Loop
				;//
Mainloop:		halt
				nop

				;// Was it a vertical blank
				ld		a, [VBFlag]
				or		a
				jr		z, Mainloop

				;// Clear the flag for next time				
				xor		a
				ld		[VBFlag], a

				;// Display the clock - Hours
				ld		a, [ClockValues + 3]
				ld		de, $9800
				call	DisplayValue
				
				;// Splitter and Minutes
				WaitForVRAM
				ld		a, 11
				ld		[de], a
				inc		de
				ld		a, [ClockValues + 2]
				call	DisplayValue

				;// Splitter and Seconds
				WaitForVRAM
				ld		a, 11
				ld		[de], a
				inc		de
				ld		a, [ClockValues + 1]
				call	DisplayValue

				;// Loop again
				jr		Mainloop

;//******************************************************************************
;//*								VERTICAL BLANK HANDLING						*
;//******************************************************************************

				;// The first thing that needs to be done is registers stored
VBHandler:		push	af
				push	hl

				;// Set a flag so we know when this has happened
				ld		a, 1
				ld		[VBFlag], a

				;// Get the start of the timer
				ld		hl, ClockValues

				;// Get the current value
				ld		a, [hl]
				inc		a
				cp		60
				jr		c, Update

				;// Over run into seconds
				xor		a
				ld		[hl+], a
				ld		a, [hl]
				inc		a
				cp		60
				jr		c, Update

				;// Over run into minutes
				xor		a
				ld		[hl+], a
				ld		a, [hl]
				inc		a
				cp		60
				jr		c, Update

				;// Over run into hours
				xor		a
				ld		[hl+], a
				ld		a, [hl]
				inc		a
				cp		24
				jr		c, Update
				xor		a

Update:			ld		[hl], a

				;// Finally - restore the registers and reti to return
				pop		hl
				pop		af
				reti

;//******************************************************************************
;//*								MISC FUNCTIONS								*
;//******************************************************************************
;// Memory Clear - Clears a chunk of memory
;//	ENTRY:
;//  A  = Value to clear to
;//  HL = Address of memory
;//  BC = Length of memory
Memory_Clear:
				;// Increase the values so we are one based
				inc		b
				inc		c
				jr		.EndCheck

				;// Clear the bytes
.ClearLoop:		ld		[hl+], a
.EndCheck:		dec		c
				jr		nz, .ClearLoop

				;// Check the high byte
				dec		b
				jr		nz, .ClearLoop

				;// Exit
				ret
				
;// MoveToVideoRam - Moves a chunk of tiles to video ram
;//	ENTRY:
;//  C = Number of tiles
;//  HL = Address of Tile Data
;//  DE = Address to move to
MoveToVideoRam:
.TileLoop:
				;// 16 Bytes per tile
				ld		b, 16

.DataLoop:
				;// Wait for video ram
				WaitForVRAM

				ld		a, [hl+]
				ld		[de], a
				inc		de
				dec		b
				jr		nz, .DataLoop										;// Loop for all 16 bytes

				dec		c
				jr		nz, .TileLoop										;// Loop for all the tiles

				ret

;// DisplayValue - Displays a in decimal
;//	ENTRY:
;//  A = Value to be displayed
;//  DE = Address to display at
DisplayValue:	ld		hl, CheatTable
				ld		c, a
				ld		b, 0
				add		hl, bc
				add		hl, bc
				
				;// We now have the ascii values
				ld		a, [hl+]
				inc		a

				;// Wait for VRAM - storing A
				WaitForVRAMStore
				ld		[de], a
				inc		de

				ld		a, [hl+]
				inc		a

				;// Wait for VRAM - storing A
				WaitForVRAMStore
				ld		[de], a
				inc		de

				ret

;// SetPalette - Sets a background palette
;//	ENTRY:
;//  A = Palette to set
;//  HL = Palette Address
SetPalette:
				add		a, a												; Get the correct palette to set (*8)
				add		a, a
				add		a, a
				add		a, $80												; Automatically increase address
				ld		[$FF68], a

				ld		b, 8												; 8 bytes per palette
				ld		c, $69												; Write Data register

.Loop:
				WaitForVRAM
				ld		a, [hl+]
				ld		[c], a
				dec		b
				jr		nz, .Loop

				ret

;//*********************************************************************************************************************
;// CHEAT LOOKUP TABLE FOR HEX - ASCII
;//*********************************************************************************************************************
CheatTable:		db		0, 0,  0, 1,  0, 2,  0, 3,  0, 4,  0, 5,  0, 6,  0, 7,  0, 8,  0, 9
				db		1, 0,  1, 1,  1, 2,  1, 3,  1, 4,  1, 5,  1, 6,  1, 7,  1, 8,  1, 9
				db		2, 0,  2, 1,  2, 2,  2, 3,  2, 4,  2, 5,  2, 6,  2, 7,  2, 8,  2, 9
				db		3, 0,  3, 1,  3, 2,  3, 3,  3, 4,  3, 5,  3, 6,  3, 7,  3, 8,  3, 9
				db		4, 0,  4, 1,  4, 2,  4, 3,  4, 4,  4, 5,  4, 6,  4, 7,  4, 8,  4, 9
				db		5, 0,  5, 1,  5, 2,  5, 3,  5, 4,  5, 5,  5, 6,  5, 7,  5, 8,  5, 9

;//*********************************************************************************************************************
;// TILE DATA
;//*********************************************************************************************************************
TileData:
	DB		%00000000, %00000000
	DB		%00000000, %00000000
	DB		%00000000, %00000000
	DB		%00000000, %00000000
	DB		%00000000, %00000000
	DB		%00000000, %00000000
	DB		%00000000, %00000000
	DB		%00000000, %00000000

	DB		%01111100, %00000000
	DB		%11000110, %00000000
	DB		%11001110, %00000000
	DB		%11010110, %00000000
	DB		%11100110, %00000000
	DB		%11000110, %00000000
	DB		%01111100, %00000000
	DB		%00000000, %00000000

	DB		%00011000, %00000000
	DB		%00111000, %00000000
	DB		%00011000, %00000000
	DB		%00011000, %00000000
	DB		%00011000, %00000000
	DB		%00011000, %00000000
	DB		%00111100, %00000000
	DB		%00000000, %00000000

	DB		%01111100, %00000000
	DB		%11000110, %00000000
	DB		%00000110, %00000000
	DB		%00011100, %00000000
	DB		%00110000, %00000000
	DB		%01100000, %00000000
	DB		%11111110, %00000000
	DB		%00000000, %00000000

	DB		%01111100, %00000000
	DB		%11000110, %00000000
	DB		%00000110, %00000000
	DB		%00111100, %00000000
	DB		%00000110, %00000000
	DB		%11000110, %00000000
	DB		%01111100, %00000000
	DB		%00000000, %00000000

	DB		%11000000, %00000000
	DB		%11000000, %00000000
	DB		%11001100, %00000000
	DB		%11001100, %00000000
	DB		%11111110, %00000000
	DB		%00001100, %00000000
	DB		%00001100, %00000000
	DB		%00000000, %00000000

	DB		%11111110, %00000000
	DB		%11000000, %00000000
	DB		%11111100, %00000000
	DB		%00000110, %00000000
	DB		%00000110, %00000000
	DB		%11000110, %00000000
	DB		%01111100, %00000000
	DB		%00000000, %00000000

	DB		%00001100, %00000000
	DB		%00011000, %00000000
	DB		%00110000, %00000000
	DB		%01111100, %00000000
	DB		%11000110, %00000000
	DB		%11000110, %00000000
	DB		%01111100, %00000000
	DB		%00000000, %00000000

	DB		%11111110, %00000000
	DB		%00000110, %00000000
	DB		%00001100, %00000000
	DB		%00011000, %00000000
	DB		%00110000, %00000000
	DB		%01100000, %00000000
	DB		%01100000, %00000000
	DB		%00000000, %00000000

	DB		%01111100, %00000000
	DB		%11000110, %00000000
	DB		%11000110, %00000000
	DB		%01111100, %00000000
	DB		%11000110, %00000000
	DB		%11000110, %00000000
	DB		%01111100, %00000000
	DB		%00000000, %00000000

	DB		%01111100, %00000000
	DB		%11000110, %00000000
	DB		%11000110, %00000000
	DB		%01111100, %00000000
	DB		%00011000, %00000000
	DB		%00110000, %00000000
	DB		%01100000, %00000000
	DB		%00000000, %00000000

	DB		%00000000, %00000000
	DB		%00000000, %00000000
	DB		%00011000, %00000000
	DB		%00011000, %00000000
	DB		%00000000, %00000000
	DB		%00011000, %00000000
	DB		%00011000, %00000000
	DB		%00000000, %00000000

Palette:
	RGB		  0,   0,   0
	RGB		255, 255, 255
	RGB		255, 255, 255
	RGB		255, 255, 255

;//******************************************************************************
;//*								VARIABLES									*
;//******************************************************************************
				SECTION "Memory", BSS[$C000]

MemoryStart:
;//* INTERRUPT VARIABLES
VBFlag:			DS		1
ClockValues:	DS		1				;// Milliseconds
				DS		1				;// Seconds
				DS		1				;// Minutes
				DS		1				;// Hours
MemoryEnd:

;//******************************************************************************
;//*							STACK SPACE										*
;//******************************************************************************

				DS		32 * 2
Stack: